@@ -11,6 +11,7 @@ import markdown2
 
 TEMPLATE_FILE_NAME = "templates/schema_doc.template.html"
 CSS_FILE_NAME = "schema_doc.css"
+JS_FILE_NAME = "schema_doc.min.js"
 
 DEFAULT_PATTERN = r"(\[Default - `([^`]+)`\])"
 DEPRECATED_PATTERN = r"\[Deprecated"
@@ -66,43 +67,48 @@ def is_deprecated_look_in_description(property_dict: Dict[str, Any]) -> bool:
 
 
 def resolve_ref(
-    property_dict: Dict[str, Any], full_schema: Dict[str, Any], schema_path: List[str]
-) -> Tuple[Dict[str, Any], List[str]]:
-    """Filter. Resolve references in the supplied property. Return the property unchanged if no references found.
+    property_dict: Dict[str, Any], full_schema: Dict[str, Any], schema_path: str
+) -> Tuple[Dict[str, Any], str]:
+    """Filter. Resolve references in the supplied property.
 
     See https://json-schema.org/understanding-json-schema/structuring.html#reuse
+
+    :param property_dict: The dict for the current property that can contain a "$ref" key
+    :param full_schema: The complete current schema, used for references inside the same file
+    :param schema_path: Path to the current schema
+    :return: The resolved schema at reference (property_dict unchanged if no references found)
+             and the path to the schema that contained the references
     """
-    if REF not in property_dict:
+    reference_path = property_dict.get(REF)
+    if not reference_path:
         return property_dict, schema_path
 
-    # Reference found, resolve the path (format "#/a/b/c" or "file.json#/a/b/c", usually "#/definitions/some name")
-    pound_split = property_dict[REF].split("#")
-    ref_file_path = [x for x in pound_split[0].split("/") if x != ""]
-    ref_anchor_path = [x for x in pound_split[1].split("/") if x != ""] if len(pound_split) > 1 else []
-    target = full_schema if ref_anchor_path else {}
+    # Reference found, resolve the path (format "#/a/b/c", "file.json#/a/b/c", or "file.json")
+    if "#" not in reference_path:
+        file_path_part = reference_path
+        anchor_part = ""
+    else:
+        file_path_part, anchor_part = reference_path.split("#", maxsplit=1)
 
     # Resolve file path portion of reference and open schema file
-    if ref_file_path:
-        target_path = schema_path[:-1]
-        for ref_path_segment in ref_file_path:
-            if ref_path_segment in (".", ""):
-                continue
-            elif ref_path_segment == "..":
-                target_path.pop()
-            else:
-                target_path.append(ref_path_segment)
-        with open(os.path.sep.join(target_path)) as schema_markdown:
+    if file_path_part:
+        target_path = os.path.abspath(os.path.join(os.path.dirname(schema_path), file_path_part))
+        with open(target_path) as schema_markdown:
             target = json.load(schema_markdown)
     else:
         target_path = schema_path
+        target = full_schema if anchor_part else {}
 
-    # Resolve anchor portion of reference
-    for ref_path_segment in ref_anchor_path:
-        if ref_path_segment in target:
-            target = target[ref_path_segment]
-        else:
-            target = property_dict
-            break
+    if anchor_part:
+        # Resolve anchor portion of reference
+        for ref_path_segment in anchor_part.split("/"):
+            if not ref_path_segment:
+                continue
+            if ref_path_segment in target:
+                target = target[ref_path_segment]
+            else:
+                target = property_dict
+                break
 
     return target, target_path
 
@@ -283,9 +289,18 @@ def get_numeric_restrictions_text(property_dict: Dict[str, Any], before_value: s
     return result if touched else ""
 
 
+def escape_property_name_for_id(property_name: str) -> str:
+    """Filter. Escape unsafe characters in a property name so that it can be used in a HTML id"""
+
+    escaped = re.sub("[^0-9a-zA-Z_,.-]", "_", property_name)
+    if not escaped[0].isalpha():
+        escaped = "a" + escaped
+    return escaped
+
+
 def generate_from_schema(
     schema: Dict[str, Any],
-    schema_path: List[str],
+    schema_path: str,
     minify: bool = False,
     deprecated_from_description: bool = False,
     default_from_description: bool = False,
@@ -300,6 +315,7 @@ def generate_from_schema(
     env.filters["get_description"] = get_description_remove_default if default_from_description else get_description
     env.filters["resolve_ref"] = resolve_ref
     env.filters["get_numeric_restrictions_text"] = get_numeric_restrictions_text
+    env.filters["escape_property_name_for_id"] = escape_property_name_for_id
     env.tests["combining"] = is_combining
     env.tests["description_short"] = is_description_short
     env.tests["deprecated"] = is_deprecated_look_in_description if deprecated_from_description else is_deprecated
@@ -308,6 +324,11 @@ def generate_from_schema(
     with open(template_path, "r") as template_fp:
         template = env.from_string(template_fp.read())
 
+    if isinstance(schema_path, list):
+        # Backward compatibility
+        schema_path = os.path.sep.join(schema_path)
+    schema_path = os.path.abspath(schema_path)
+
     rendered = template.render(schema=schema, schema_path=schema_path, expand_buttons=expand_buttons)
     if minify:
         rendered = htmlmin.minify(rendered)
@@ -321,20 +342,21 @@ def generate_from_filename(
     minify: bool = True,
     deprecated_from_description: bool = False,
     default_from_description: bool = False,
+    copy_css: bool = True,
+    copy_js: bool = True,
 ) -> None:
     with open(schema_file_name) as schema_markdown:
         schema = json.load(schema_markdown)
-        schema_path = os.path.abspath(schema_file_name).split(os.path.sep)
 
     rendered_schema_doc = generate_from_schema(
         schema,
-        schema_path,
+        os.path.abspath(schema_file_name),
         minify=minify,
         deprecated_from_description=deprecated_from_description,
         default_from_description=default_from_description,
     )
 
-    copy_css_to_target(result_file_name)
+    copy_css_and_js_to_target(result_file_name, copy_css, copy_js)
 
     with open(result_file_name, "w") as result_schema_doc:
         result_schema_doc.write(rendered_schema_doc)
@@ -347,34 +369,46 @@ def generate_from_file_object(
     deprecated_from_description: bool,
     default_from_description: bool,
     expand_buttons: bool,
+    copy_css: bool = True,
+    copy_js: bool = True,
 ) -> None:
     """Generate the JSON schema documentation from opened file objects for both input and output files. The
     result_file should be opened in write mode.
     """
-    schema_path = os.path.abspath(schema_file.name).split(os.path.sep)
     result = generate_from_schema(
         json.load(schema_file),
-        schema_path,
+        os.path.abspath(schema_file.name),
         minify,
         deprecated_from_description,
         default_from_description,
         expand_buttons,
     )
 
-    copy_css_to_target(result_file.name)
+    copy_css_and_js_to_target(result_file.name, copy_css, copy_js)
 
     result_file.write(result)
 
 
-def copy_css_to_target(result_file_path: str) -> None:
-    """Copy the CSS file needed to display the resulting page to the directory containing the result file"""
+def copy_css_and_js_to_target(result_file_path: str, copy_css: bool, copy_js: bool) -> None:
+    """Copy the CSS and JS files needed to display the resulting page to the directory containing the result file"""
+    files_to_copy = []
+    if copy_css:
+        files_to_copy.append(CSS_FILE_NAME)
+    if copy_js:
+        files_to_copy.append(JS_FILE_NAME)
+    if not files_to_copy:
+        return
+
     target_directory = os.path.dirname(result_file_path)
-    css_directory = os.path.dirname(__file__)
-    if target_directory != css_directory:
+    source_directory = os.path.dirname(__file__)
+    if target_directory == source_directory:
+        return
+
+    for file_to_copy in files_to_copy:
         try:
-            shutil.copy(os.path.join(css_directory, CSS_FILE_NAME), os.path.join(target_directory, CSS_FILE_NAME))
+            shutil.copy(os.path.join(source_directory, file_to_copy), os.path.join(target_directory, file_to_copy))
         except shutil.SameFileError:
-            print(f"Not copying {CSS_FILE_NAME} to {os.path.abspath(target_directory)}, file already exists")
+            print(f"Not copying {file_to_copy} to {os.path.abspath(target_directory)}, file already exists")
 
 
 @click.command()
@@ -388,6 +422,8 @@ def copy_css_to_target(result_file_path: str) -> None:
     "--default-from-description", is_flag=True, help="Look in the description to find an attribute default value"
 )
 @click.option("--expand-buttons", is_flag=True, help="Add 'Expand all' and 'Collapse all' buttons at the top")
+@click.option("--copy-css/--no-copy-css", default=True, help=f"Copy {CSS_FILE_NAME} to the folder of the result_file")
+@click.option("--copy-js/--no-copy-js", default=True, help=f"Copy {JS_FILE_NAME} to the folder of the result_file")
 def main(
     schema_file: TextIO,
     result_file: TextIO,
@@ -395,9 +431,18 @@ def main(
     deprecated_from_description: bool,
     default_from_description: bool,
     expand_buttons: bool,
+    copy_css: bool,
+    copy_js: bool,
 ) -> None:
     generate_from_file_object(
-        schema_file, result_file, minify, deprecated_from_description, default_from_description, expand_buttons
+        schema_file,
+        result_file,
+        minify,
+        deprecated_from_description,
+        default_from_description,
+        expand_buttons,
+        copy_css,
+        copy_js,
     )
 
 
