@@ -7,11 +7,11 @@ from bs4 import BeautifulSoup
 from json_schema_for_humans.generate import generate_from_schema
 
 
-def _get_test_case(name: str) -> Tuple[Dict[str, Any], List[str]]:
+def _get_test_case(name: str) -> Tuple[Dict[str, Any], str]:
     """Get the loaded JSON schema for a test case"""
     test_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "cases", f"{name}.json"))
-    with open(test_path) as test_case_file:
-        return (json.load(test_case_file), test_path.split(os.path.sep))
+    with open(test_path, encoding="utf-8") as test_case_file:
+        return (json.load(test_case_file), os.path.abspath(test_path))
 
 
 def _generate_case(case_name: str, find_deprecated: bool = False, find_default: bool = False) -> BeautifulSoup:
@@ -42,7 +42,7 @@ def _assert_property_names(soup: BeautifulSoup, property_names: List[str]) -> No
 def _assert_title(soup: BeautifulSoup, title: str) -> None:
     """Assert the result file contains the provided title"""
     assert soup.head.title.string == title
-    assert soup.head.h1.string == title
+    assert soup.body.h1.string == title
 
 
 def _assert_descriptions(soup: BeautifulSoup, descriptions: List[str]) -> None:
@@ -158,6 +158,7 @@ def test_references() -> None:
     _assert_descriptions(
         soup,
         [
+            "Testing $ref",
             "Description for object_def/items/propertyA",
             "Description for array_def",
             "Description for string_def",
@@ -193,7 +194,14 @@ def test_array() -> None:
     soup = _generate_case("array")
 
     _assert_property_names(soup, ["fruits", "vegetables", "veggieName", "veggieLike"])
-    _assert_descriptions(soup, ["The name of the vegetable.", "Do I like this vegetable?"])
+    _assert_descriptions(
+        soup,
+        [
+            "A representation of a person, company, organization, or place",
+            "The name of the vegetable.",
+            "Do I like this vegetable?",
+        ],
+    )
     _assert_types(soup, ["array of string", "string", "array", "object", "string", "boolean"])
     _assert_required(soup, [False, False, True, True])
 
@@ -202,7 +210,7 @@ def test_array_advanced():
     """Test rendering a schema that uses minItems, maxItems, and uniqueItems for arrays"""
     soup = _generate_case("array_advanced")
 
-    _assert_descriptions(soup, ["5 to 8 fruits that you like"])
+    _assert_descriptions(soup, ["A little food fun", "5 to 8 fruits that you like"])
     _assert_property_names(soup, ["fruits", "vegetables"])
     _assert_const(soup, ["eggplant"])
     _assert_required(soup, [False] * 2)
@@ -274,3 +282,14 @@ def test_deprecated_not_in_description() -> None:
     soup = _generate_case("deprecated", find_deprecated=False)
 
     _assert_deprecated(soup, [False] * 3)
+
+
+def test_with_special_chars() -> None:
+    soup = _generate_case("with_special_chars", find_deprecated=False)
+
+    _assert_property_names(soup, ["prénom", "nomDeFamille", "âge", "0 de quoi d'autre"])
+
+    buttons = soup.find_all("button", attrs={"aria-controls": True})
+    expected_targets = ["#pr_nom", "#nomDeFamille", "#a_ge", "#a0_de_quoi_d_autre"]
+    for i, expected_target in enumerate(expected_targets):
+        assert buttons[i].attrs["data-target"] == expected_target
